/* ownop.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2004 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: ownop.cc,v 1.33 2005/02/07 07:09:35 ralf Exp $ */

#include "ownop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"
#include "filereq.h"

const char *OwnOp::name="OwnOp";

OwnOp::OwnOp() : FunctionProto()
{
  separate_each_entry=false;
  recursive=false;
  inbackground = false;
  take_dirs = false;

  ownstart=OWNOP_START_NORMAL;

  com_str=dupstring("");
  view_str=dupstring("");
  hasConfigure = true;
  
  dontcd = false;
}

OwnOp::~OwnOp()
{
  _freesafe(com_str);
  _freesafe(view_str);
}

OwnOp*
OwnOp::duplicate() const
{
  OwnOp *ta=new OwnOp();
  ta->separate_each_entry=separate_each_entry;
  ta->recursive=recursive;
  ta->inbackground = inbackground;
  ta->take_dirs = take_dirs;
  ta->dontcd = dontcd;

  ta->ownstart=ownstart;

  _freesafe(ta->com_str);
  _freesafe(ta->view_str);
  ta->com_str=dupstring(com_str);
  ta->view_str=dupstring(view_str);
  return ta;
}

bool
OwnOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
OwnOp::getName()
{
  return name;
}

int
OwnOp::run( WPUContext *wpu, ActionMessage *msg )
{
  ListerMode *lm1;
  am=msg;
  use_wpu = wpu;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1 = msg->getWorker()->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      endlister = msg->getWorker()->getOtherLister(startlister);
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodeown();
    }
  } else normalmodeown();
  return 0;
}

bool
OwnOp::save(Datei *fh)
{
  if ( fh == NULL ) return false;

  WConfig::configPutPairBool( fh, "separateeachentry", separate_each_entry );
  WConfig::configPutPairBool( fh, "recursive", recursive );
  switch ( ownstart ) {
    case OWNOP_START_IN_TERMINAL:
      WConfig::configPutPair( fh, "start", "terminal" );
      break;
    case OWNOP_START_IN_TERMINAL_AND_WAIT4KEY:
      WConfig::configPutPair( fh, "start", "terminalwait" );
      break;
    case OWNOP_SHOW_OUTPUT:
      WConfig::configPutPair( fh, "start", "showoutput" );
      break;
    default:
      WConfig::configPutPair( fh, "start", "normal" );
      break;
  }
  WConfig::configPutPairString( fh, "com", com_str );
  WConfig::configPutPairString( fh, "viewstr", view_str );
  
  WConfig::configPutPairBool( fh, "inbackground", inbackground );
  WConfig::configPutPairBool( fh, "takedirs", take_dirs );
  WConfig::configPutPairBool( fh, "dontcd", dontcd );
  return true;
}

const char *
OwnOp::getDescription()
{
  return catalog.getLocaleCom(28);
}

int
OwnOp::normalmodeown()
{
  class NM_externorder extorder;
  NormalMode *nm1=NULL,
             *nm2=NULL;
  ListerMode *lm1=NULL;
  NM_specialsourceExt *specialsource=NULL;
  
  if(am->mode==am->AM_MODE_DNDACTION) {
    lm1 = am->dndmsg->getSourceMode();
    if(lm1==NULL) return 1;
    if(lm1->isType("NormalMode")!=true) return 1;
    nm1=(NormalMode*)lm1;
    lm1 = am->dndmsg->getDestMode();
    if(lm1!=NULL)
      if(lm1->isType("NormalMode")==true)
        nm2=(NormalMode*)lm1;
  } else {
    if(startlister==NULL) return 1;
    lm1=startlister->getActiveMode();
    if(lm1==NULL) return 1;
    if(lm1->isType("NormalMode")!=true) return 1;
    nm1=(NormalMode*)lm1;
    if(endlister!=NULL) {
      lm1=endlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          nm2=(NormalMode*)lm1;
    }
  }
  
  extorder.wpu = use_wpu;
  extorder.destmode=NULL;
  if(am->mode==am->AM_MODE_ONLYACTIVE)
    extorder.source=extorder.NM_ONLYACTIVE;
  else if(am->mode==am->AM_MODE_DNDACTION) {
    // insert DND-element into list
    extorder.source=extorder.NM_SPECIAL;
    extorder.sources=new std::list<NM_specialsourceExt*>;
    specialsource = new NM_specialsourceExt( am->dndmsg->getFE() );
    specialsource->row=-1;
    extorder.sources->push_back(specialsource);
    extorder.destmode=nm2;
  } else if(am->mode==am->AM_MODE_SPECIAL) {
    extorder.source=extorder.NM_SPECIAL;
    extorder.sources=new std::list<NM_specialsourceExt*>;
    specialsource = new NM_specialsourceExt( am->getFE() );
    specialsource->row=-1;
    extorder.sources->push_back(specialsource);
  } else
    extorder.source=extorder.NM_ALLENTRIES;

  extorder.separate_each_entry=separate_each_entry;
  extorder.recursive=recursive;
  extorder.inbackground = inbackground;
  extorder.take_dirs = take_dirs;
  switch(ownstart) {
    case OWNOP_START_IN_TERMINAL:
      extorder.extstart=extorder.NM_EXT_START_IN_TERMINAL;
      break;
    case OWNOP_START_IN_TERMINAL_AND_WAIT4KEY:
      extorder.extstart=extorder.NM_EXT_START_IN_TERMINAL_AND_WAIT4KEY;
      break;
    case OWNOP_SHOW_OUTPUT:
      extorder.extstart=extorder.NM_EXT_SHOW_OUTPUT;
      break;
    default:
      extorder.extstart=extorder.NM_EXT_START_NORMAL;
      break;
  }
  extorder.com_str=NULL;
  extorder.view_str=NULL;
  if(strlen(com_str)>0) extorder.com_str=com_str;
  if(strlen(view_str)>0) extorder.view_str=view_str;

  extorder.dontcd = dontcd;

  nm1->externprog(&extorder);

  if(extorder.source==extorder.NM_SPECIAL) {
    if ( specialsource != NULL ) delete specialsource;
    delete extorder.sources;
  }

  return 0;
}

int
OwnOp::configure()
{
  AGUIX *aguix = Worker::getAGUIX();
  Button *okb,*cb,*fb, *comreqb, *comreqb2;
  StringGadget *sg[2];
  AWindow *win;
  ChooseButton *scb,*rcb, *ibcb, *tdcb, *dcdcb;
  CycleButton *cyb;
  Text *ttext;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  Requester *req;
  FileRequester *freq;
  const char *textstr;
  char *buttonstr,*tstr;
  int erg;
  std::string str1;
  
  req=new Requester(aguix);
  freq = new FileRequester( aguix );
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);

  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(28))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(28));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);

  scb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,(separate_each_entry==true)?1:0,
                                               catalog.getLocale(329),LABEL_RIGHT,1,0));
  y+=scb->getHeight()+5;
  tw=scb->getWidth()+10;
  if(tw>w) w=tw;

  rcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,(recursive==true)?1:0,
                                               catalog.getLocale(330),LABEL_RIGHT,1,0));
  y+=rcb->getHeight()+5;
  tw=rcb->getWidth()+10;
  if(tw>w) w=tw;

  tdcb = (ChooseButton*)win->add( new ChooseButton( aguix, x, y, ( take_dirs == true ) ? 1 : 0,
                                                    catalog.getLocale( 513 ), LABEL_RIGHT, 1, 0 ) );
  y += tdcb->getHeight() + 5;
  tw = tdcb->getWidth() + 10;
  if ( tw > w ) w = tw;

  dcdcb = (ChooseButton*)win->add( new ChooseButton( aguix, x, y, ( dontcd == true ) ? 1 : 0,
						     catalog.getLocale( 617 ), LABEL_RIGHT, 1, 0 ) );
  y += dcdcb->getHeight() + 5;
  tw = dcdcb->getWidth() + 10;
  if ( tw > w ) w = tw;

  ibcb = (ChooseButton*)win->add( new ChooseButton( aguix,
                                                    x,
                                                    y,
                                                    ( inbackground == true ) ? 1 : 0,
                                                    catalog.getLocale( 435 ),
                                                    LABEL_RIGHT,
                                                    1,
                                                    0 ) );
  y+=ibcb->getHeight()+5;
  tw=ibcb->getWidth()+10;
  if(tw>w) w=tw;

  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(331),1));
  x+=ttext->getWidth()+5;
  cyb=(CycleButton*)win->add(new CycleButton(aguix,x,y,100,1,0,0));
  cyb->addOption(catalog.getLocale(332));
  cyb->addOption(catalog.getLocale(333));
  cyb->addOption(catalog.getLocale(334));
  cyb->addOption(catalog.getLocale(335));
  cyb->resize(cyb->getMaxSize(),cyb->getHeight());
  switch(ownstart) {
    case OWNOP_START_IN_TERMINAL:
      cyb->setOption(1);
      break;
    case OWNOP_START_IN_TERMINAL_AND_WAIT4KEY:
      cyb->setOption(2);
      break;
    case OWNOP_SHOW_OUTPUT:
      cyb->setOption(3);
      break;
    default:
      cyb->setOption(0);
      break;
  }
  
  y+=cyb->getHeight()+5;
  tw=x+cyb->getWidth()+5;
  if(tw>w) w=tw;
  x=5;

  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(336),1));
  x+=ttext->getWidth()+5;
  comreqb = (Button*)win->add( new Button( aguix, x, y, catalog.getLocale( 458 ), 1, 0, 0 ) );
  x += comreqb->getWidth();

  tw=w-x-5;
  if ( tw < ( 20 * aguix->getCharWidth() ) ) tw = 20 * aguix->getCharWidth();
  sg[0]=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,com_str,0));
  
  fb=(Button*)win->add(new Button(aguix,x+sg[0]->getWidth(),y,sg[0]->getHeight(),
                                  sg[0]->getHeight(),"O",1,0,0));
  comreqb->resize( comreqb->getWidth(), sg[0]->getHeight() );
  
  y+=sg[0]->getHeight()+5;
  tw=sg[0]->getWidth()+5+x;
  if(tw>w) w=tw;
  x=5;
  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(337),1));
  x+=ttext->getWidth()+5;
  comreqb2 = (Button*)win->add( new Button( aguix, x, y, catalog.getLocale( 458 ), 1, 0, 0 ) );
  x += comreqb2->getWidth();
  
  tw=w-x-5;
  if ( tw < ( 20 * aguix->getCharWidth() ) ) tw = 20 * aguix->getCharWidth();
  sg[1]=(StringGadget*)win->add(new StringGadget(aguix,x,y,tw,view_str,0));
  comreqb2->resize( comreqb2->getWidth(), sg[1]->getHeight() );
  
  y+=sg[1]->getHeight()+5;
  tw=sg[1]->getWidth()+5+x;
  if(tw>w) w=tw;
  x=5;

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  sg[0]->resize(w-5-sg[0]->getX()-sg[0]->getHeight(),sg[0]->getHeight());
  sg[1]->resize(w-5-sg[1]->getX(),sg[1]->getHeight());
  fb->move(w-5-fb->getWidth(),fb->getY());

  h=y;
  win->setDoTabCycling( true );
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) {
            if(NormalMode::isCorrectViewProg(sg[1]->getText())==false) {
              // show requester
              textstr=catalog.getLocale(312);
              buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
                                          strlen(catalog.getLocale(314))+1);
              sprintf(buttonstr,"%s|%s",catalog.getLocale(313),
                                        catalog.getLocale(314));
              erg=req->request(catalog.getLocale(125),textstr,buttonstr);
              _freesafe(buttonstr);
              if(erg==0) {
                sg[0]->deactivate();
                sg[1]->activate();
              } else {
                sg[1]->setText("");
              }
            } else endmode=0;
          } else if(msg->button.button==cb) endmode=1;
          else if(msg->button.button==fb) {
            tstr=getFlag();
            if(tstr!=NULL) {
              const char *newstr1=sg[0]->getText();
              char *newstr2;
              int pos=sg[0]->getCursor();
              newstr2=(char*)_allocsafe(strlen(newstr1)+strlen(tstr)+1);
              strncpy(newstr2,newstr1,pos);
              strcpy(newstr2+pos,tstr);
              strcpy(newstr2+pos+strlen(tstr),newstr1+pos);
              sg[0]->setText(newstr2);
              _freesafe(newstr2);
              _freesafe(tstr);
            }
          } else if ( msg->button.button == comreqb ) {
            if ( freq->request_entry( catalog.getLocale( 265 ),
                                      NULL,
                                      catalog.getLocale( 11 ),
                                      catalog.getLocale( 8 ),
                                      catalog.getLocale( 265 ),
                                      true ) > 0 ) {
              str1 = freq->getLastEntryStr();
              if ( str1.length() > 0 ) {
                const char *newstr1 = sg[ 0 ]->getText();
                char *newstr2;
                int pos = sg[ 0 ]->getCursor();

                newstr2 = (char*)_allocsafe( strlen( newstr1 ) + strlen( str1.c_str() ) + 1 );
                strncpy( newstr2, newstr1, pos );
                strcpy( newstr2 + pos, str1.c_str() );
                strcpy( newstr2 + pos + strlen( str1.c_str() ), newstr1 + pos );
                sg[ 0 ]->setText( newstr2 );
                _freesafe( newstr2 );
              }
            }
          } else if ( msg->button.button == comreqb2 ) {
            if ( freq->request_entry( catalog.getLocale( 265 ),
                                      NULL,
                                      catalog.getLocale( 11 ),
                                      catalog.getLocale( 8 ),
                                      catalog.getLocale( 265 ),
                                      true ) > 0 ) {
              str1 = freq->getLastEntryStr();
              if ( str1.length() > 0 ) {
                const char *newstr1 = sg[ 1 ]->getText();
                char *newstr2;
                int pos = sg[ 1 ]->getCursor();

                newstr2 = (char*)_allocsafe( strlen( newstr1 ) + strlen( str1.c_str() ) + 1 );
                strncpy( newstr2, newstr1, pos );
                strcpy( newstr2 + pos, str1.c_str() );
                strcpy( newstr2 + pos + strlen( str1.c_str() ), newstr1 + pos );
                sg[ 1 ]->setText( newstr2 );
                _freesafe( newstr2 );
              }
            }
          }
          break;
	case AG_STRINGGADGET_DEACTIVATE:
          if(msg->stringgadget.sg==sg[1]) {
            if(NormalMode::isCorrectViewProg(sg[1]->getText())==false) {
              // show requester
              textstr=catalog.getLocale(312);
              buttonstr=(char*)_allocsafe(strlen(catalog.getLocale(313))+1+
                                          strlen(catalog.getLocale(314))+1);
              sprintf(buttonstr,"%s|%s",catalog.getLocale(313),
                                        catalog.getLocale(314));
              erg=req->request(catalog.getLocale(125),textstr,buttonstr);
              _freesafe(buttonstr);
              if(erg==0) {
                sg[0]->deactivate();
                sg[1]->activate();
              } else {
                sg[1]->setText("");
              }
            }
          }
	  break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    separate_each_entry=(scb->getState()==1)?true:false;
    recursive=(rcb->getState()==1)?true:false;
    take_dirs = ( tdcb->getState() == 1 ) ? true : false;
    dontcd = ( dcdcb->getState() == 1 ) ? true : false;
    inbackground = ( ibcb->getState() == 1 ) ? true : false;
    _freesafe(com_str);
    _freesafe(view_str);
    com_str=dupstring(sg[0]->getText());
    view_str=dupstring(sg[1]->getText());
    switch(cyb->getSelectedOption()) {
      case 1:
        ownstart=OWNOP_START_IN_TERMINAL;
        break;
      case 2:
        ownstart=OWNOP_START_IN_TERMINAL_AND_WAIT4KEY;
        break;
      case 3:
        ownstart=OWNOP_SHOW_OUTPUT;
        break;
      default:
        ownstart=OWNOP_START_NORMAL;
        break;
    }
  }
  
  win->close();
  delete win;
  delete req;
  delete freq;

  return endmode;
}

char*
OwnOp::getFlag()
{
  AGUIX *aguix = Worker::getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  FieldListView *lv;
  Text *ttext;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  char *returnstr;
  Button *b2[2];
  int i, trow, nr;
  
  w=400;
  h=300;
  x=y=5;
  win=new AWindow(aguix);
  win->create(NULL,10,10,w,h,0,catalog.getLocale(338));

  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(338),1));
  y+=ttext->getHeight()+5;

  lv = (FieldListView*)win->add( new FieldListView( aguix,
                                                    x,
                                                    y,
                                                    w - 2 * x,
                                                    200,
                                                    0 ) );
  y+=lv->getHeight()+5;
  lv->setHBarState(2);
  lv->setVBarState(2);

  lv->setNrOfFields( 3 );
  lv->setFieldWidth( 1, 3 );
  
  nr = sizeof( ownop_flags ) / sizeof( ownop_flags[0] );

  for( i=0;i<nr;i++) {
    trow = lv->addRow();
    lv->setText( trow, 0, ownop_flags[ i ].flag );
    lv->setText( trow, 2, catalog.getLocaleFlag( ownop_flags[ i ].catcode ) );
    lv->setPreColors( trow, FieldListView::PRECOLOR_ONLYACTIVE );
    lv->setData( trow, i );
  }

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;

  b2[0] = okb;
  b2[1] = cb;
  
  h=y;
  win->setDoTabCycling( true );
  win->resize(w,h);
  //win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
        case AG_SIZECHANGED:
          AGUIX::scaleElementsW( msg->size.neww, 5, 10, -1, false, false, (GUIElement**)b2, NULL, 2 );
          for ( i = 0; i < 2; i++ ) {
            b2[i]->move( b2[i]->getX(), msg->size.newh - 5 - b2[i]->getHeight() );
          }
          lv->resize( msg->size.neww - 10, b2[0]->getY() - 5 - lv->getY() );
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  returnstr=NULL;
  if(endmode==0) {
    // ok
    trow = lv->getActiveRow();
    if ( lv->isValidRow( trow ) == true ) {
      nr = lv->getData( trow );
      returnstr = dupstring( ownop_flags[nr].flag );
    }
  }
  
  win->close();
  delete win;

  return returnstr;
}

void OwnOp::setSeparateEachEntry(bool nv)
{
  separate_each_entry=nv;
}

void OwnOp::setRecursive(bool nv)
{
  recursive=nv;
}

void OwnOp::setOwnStart(ownstart_t nv)
{
  ownstart=nv;
}

void OwnOp::setComStr(const char* nv)
{
  if ( nv == NULL ) return;
  _freesafe(com_str);
  com_str=dupstring(nv);
}

void OwnOp::setViewStr( const char* nv )
{
  if ( nv == NULL ) return;
  _freesafe( view_str );
  view_str = dupstring( nv );
}

void OwnOp::setInBackground( bool nv )
{
  inbackground = nv;
}

void OwnOp::setTakeDirs( bool nv )
{
  take_dirs = nv;
}

void OwnOp::setDontCD( bool nv )
{
  dontcd = nv;
}
